--[[
********************************
Licence: GPL
(C)2012 rudi, Version: 0.2

CORDIC TEST
********************************
@title cordic test V0.2
]]

-- Int2Str(value[,x10^dpow:default=0[, unit:string][, fix:number]])
function Int2Str(val, dpow, ...)
    local _dpow, _sign, _val, _unit, _fix = dpow or 0, (val < 0) and "-" or "", tostring(math.abs(val))
    for i = 1, select('#', ...) do
        local _arg = select(i, ...)
        if not _unit and type(_arg) == "string" and #_arg > 0 then _unit = _arg
        elseif not _fix and type(_arg) == "number" and _arg >= 0 then _fix = _arg
        end
    end
    _val = (_dpow < 0) and string.rep("0", 1 - #_val - _dpow) .. _val or _val .. string.rep("0", _dpow)
    local _int, _frac = string.match(_val, "^([%d]+)(" .. string.rep("%d", -_dpow) .. ")$")
    _frac = _fix and string.sub((_frac or "") .. string.rep("0", _fix), 1, _fix) or _frac
    _frac = (_frac and type(_frac) == "string" and #_frac > 0) and "." .. _frac or ""
    return  string.format("%s%s%s%s", _sign, _int, _frac, _unit or "")
end

-----MAIN-----
set_console_layout(0,0,39,14)

cordic = require("cordicLib")

print(string.format("CORDIC LIBRARY Version %s", cordic.version))
print()

print("Konstanten:")
print(string.format("Multiplikator = %s", Int2Str(cordic.SCALE)))
print(string.format("2*PI = %s", Int2Str(cordic.PI2, -3)))
print(string.format("PI = %s", Int2Str(cordic.PI, -3)))
print(string.format("PI/2 = %s", Int2Str(cordic.PI_2, -3)))
print()

print("Funktionen:")
winkel_deg = 30 * cordic.SCALE
winkel_rad = cordic.deg2rad(winkel_deg)
print(string.format("DEG(%s) -> RAD(%s)", Int2Str(winkel_deg, -3, 0), Int2Str(winkel_rad, -3)))
_winkel = cordic.rad2deg(winkel_rad)
print(string.format("RAD(%s) -> DEG(%s)", Int2Str(winkel_rad, -3), Int2Str(_winkel, -3)))
print(string.format("4/3 = %s", Int2Str(cordic.integer_mul_div(cordic.SCALE, 4 * cordic.SCALE, 3 * cordic.SCALE), -3))) -- = 1 * 4 / 3
print()

print("Winkelfunktionen im Bogenma (RAD):")
-- sinr, cosr, tanr
sin_rad = cordic.sinr(winkel_rad)
cos_rad = cordic.cosr(winkel_rad)
tan_rad = cordic.tanr(winkel_rad)
print(string.format("SINR(%s) = %s", Int2Str(winkel_rad, -3), Int2Str(sin_rad, -3)))
print(string.format("COSR(%s) = %s", Int2Str(winkel_rad, -3), Int2Str(cos_rad, -3)))
print(string.format("TANR(%s) = %s", Int2Str(winkel_rad, -3), Int2Str(tan_rad, -3)))
print()

-- asinr, acosr, atanr
asin_rad = cordic.asinr(sin_rad)
acos_rad = cordic.acosr(cos_rad)
atan_rad = cordic.atanr(tan_rad)
print(string.format("ASINR(%s) = %s", Int2Str(sin_rad, -3), Int2Str(asin_rad, -3)))
print(string.format("ACOSR(%s) = %s", Int2Str(cos_rad, -3), Int2Str(acos_rad, -3)))
print(string.format("ATANR(%s) = %s", Int2Str(tan_rad, -3), Int2Str(atan_rad, -3)))
print()

seitex = 4 * cordic.SCALE --4
seitey = 3 * cordic.SCALE --3
--recr to polr
vektor, winkel = cordic.polr(seitex, seitey)
print(string.format("RECR(%s, %s) = POLR(%s, %s)", Int2Str(seitex, -3), Int2Str(seitey, -3), Int2Str(vektor, -3), Int2Str(winkel, -3)))
--polr to recr
seitex, seitey = cordic.recr(vektor, winkel)
print(string.format("POLR(%s, %s) = RECR(%s, %s)", Int2Str(vektor, -3), Int2Str(winkel, -3), Int2Str(seitex, -3), Int2Str(seitey, -3)))
print()

print("Winkelfunktionen in Grad (DEG):")
-- sind, cosd, tand
sin_deg = cordic.sind(winkel_deg)
cos_deg = cordic.cosd(winkel_deg)
tan_deg = cordic.tand(winkel_deg)
print(string.format("SIND(%s) = %s", Int2Str(winkel_deg, -3, 0), Int2Str(sin_deg, -3)))
print(string.format("COSD(%s) = %s", Int2Str(winkel_deg, -3, 0), Int2Str(cos_deg, -3)))
print(string.format("TAND(%s) = %s", Int2Str(winkel_deg, -3, 0), Int2Str(tan_deg, -3)))
print()

-- asind, acosd, atand
asin_deg = cordic.asind(sin_deg)
acos_deg = cordic.acosd(cos_deg)
atan_deg = cordic.atand(tan_deg)
print(string.format("ASIND(%s) = %s", Int2Str(sin_deg, -3), Int2Str(asin_deg, -3)))
print(string.format("ACOSD(%s) = %s", Int2Str(cos_deg, -3), Int2Str(acos_deg, -3)))
print(string.format("ATAND(%s) = %s", Int2Str(tan_deg, -3), Int2Str(atan_deg, -3)))
print()

seitex = 4 * cordic.SCALE --4
seitey = 3 * cordic.SCALE --3
--recd to pold
vektor, winkel = cordic.pold(seitex, seitey)
print(string.format("RECD(%s, %s) = POLD(%s, %s)", Int2Str(seitex, -3), Int2Str(seitey, -3), Int2Str(vektor, -3), Int2Str(winkel, -3)))
--pold to recd
seitex, seitey = cordic.recd(vektor, winkel)
print(string.format("POLD(%s, %s) = RECD(%s, %s)", Int2Str(vektor, -3), Int2Str(winkel, -3), Int2Str(seitex, -3), Int2Str(seitey, -3)))
